@@ -8,11 +8,18 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #' to errors and returns several variables.
 #'
 #' The function estimates Complex Exponential Smoothing in the state-space 2
-#' described in Svetunkov, Kourentzes (2015) with the information potential
+#' described in Svetunkov, Kourentzes (2017) with the information potential
 #' equal to the approximation error.  The estimation of initial states of xt is
 #' done using backcast.
 #'
-#' @param data Either numeric vector or time series vector.
+#' @template ssBasicParam
+#' @template ssAdvancedParam
+#' @template ssInitialParam
+#' @template ssAuthor
+#' @template ssKeywords
+#'
+#' @template ssCESRef
+#'
 #' @param seasonality The type of seasonality used in CES. Can be: \code{none}
 #' - No seasonality; \code{simple} - Simple seasonality, using lagged CES
 #' (based on \code{t-m} observation, where \code{m} is the seasonality lag);
@@ -22,10 +29,6 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #' seasonality, depending on the data). First letter can be used instead of
 #' full words.  Any seasonal CES can only be constructed for time series
 #' vectors.
-#' @param initial Can be either character or a vector of initial states. If it
-#' is character, then it can be \code{"optimal"}, meaning that the initial
-#' states are optimised, or \code{"backcasting"}, meaning that the initials are
-#' produced using backcasting procedure (advised for data with high frequency).
 #' @param A First complex smoothing parameter. Should be a complex number.
 #'
 #' NOTE! CES is very sensitive to A and B values so it is advised either to
@@ -33,87 +36,6 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #' @param B Second complex smoothing parameter. Can be real if
 #' \code{seasonality="partial"}. In case of \code{seasonality="full"} must be
 #' complex number.
-#' @param ic Information criterion to use in model selection.
-#' @param cfType Type of Cost Function used in optimization. \code{cfType} can
-#' be: \code{MSE} (Mean Squared Error), \code{MAE} (Mean Absolute Error),
-#' \code{HAM} (Half Absolute Moment), \code{MLSTFE} - Mean Log Squared Trace
-#' Forecast Error, \code{MSTFE} - Mean Squared Trace Forecast Error and
-#' \code{MSEh} - optimisation using only h-steps ahead error. If
-#' \code{cfType!="MSE"}, then likelihood and model selection is done based on
-#' equivalent \code{MSE}. Model selection in this cases becomes not optimal.
-#'
-#' There are also available analytical approximations for multistep functions:
-#' \code{aMSEh}, \code{aMSTFE} and \code{aMLSTFE}. These can be useful in cases
-#' of small samples.
-#' @param h The forecasting horizon.
-#' @param holdout If \code{TRUE}, holdout sample of size \code{h} is taken from
-#' the end of the data.
-#' @param intervals Type of intervals to construct. This can be:
-#'
-#' \itemize{
-#' \item \code{none}, aka \code{n} - do not produce prediction intervals.
-#' \item \code{parametric}, \code{p} - use state-space structure of ETS. In
-#' case of mixed models this is done using simulations, which may take longer
-#' time than for the pure additive and pure multiplicative models.
-#' \item \code{semiparametric}, \code{sp} - intervals based on covariance
-#' matrix of 1 to h steps ahead errors and assumption of normal / log-normal
-#' distribution (depending on error type).
-#' \item \code{nonparametric}, \code{np} - intervals based on values from a
-#' quantile regression on error matrix (see Taylor and Bunn, 1999). The model
-#' used in this process is e[j] = a j^b, where j=1,..,h.
-#' %\item Finally \code{asymmetric} are based on half moment of distribution.
-#' }
-#'
-#' The parameter also accepts \code{TRUE} and \code{FALSE}. Former means that
-#' parametric intervals are constructed, while latter is equivalent to
-#' \code{none}.
-#' @param level Confidence level. Defines width of prediction interval.
-#' @param intermittent Defines type of intermittent model used. Can be: 1.
-#' \code{none}, meaning that the data should be considered as non-intermittent;
-#' 2. \code{fixed}, taking into account constant Bernoulli distribution of
-#' demand occurancies; 3. \code{croston}, based on Croston, 1972 method with
-#' SBA correction; 4. \code{tsb}, based on Teunter et al., 2011 method. 5.
-#' \code{auto} - automatic selection of intermittency type based on information
-#' criteria. The first letter can be used instead. 6. \code{"sba"} -
-#' Syntetos-Boylan Approximation for Croston's method (bias correction)
-#' discussed in Syntetos and Boylan, 2005.
-#' @param bounds What type of bounds to use for smoothing parameters
-#' ("admissible" or "usual"). The first letter can be used instead of the whole
-#' word.
-#' @param silent If \code{silent="none"}, then nothing is silent, everything is
-#' printed out and drawn. \code{silent="all"} means that nothing is produced or
-#' drawn (except for warnings). In case of \code{silent="graph"}, no graph is
-#' produced. If \code{silent="legend"}, then legend of the graph is skipped.
-#' And finally \code{silent="output"} means that nothing is printed out in the
-#' console, but the graph is produced. \code{silent} also accepts \code{TRUE}
-#' and \code{FALSE}. In this case \code{silent=TRUE} is equivalent to
-#' \code{silent="all"}, while \code{silent=FALSE} is equivalent to
-#' \code{silent="none"}. The parameter also accepts first letter of words ("n",
-#' "a", "g", "l", "o").
-#' @param xreg Vector (either numeric or time series) or matrix (or data.frame)
-#' of exogenous variables that should be included in the model. If matrix
-#' included than columns should contain variables and rows - observations. Note
-#' that \code{xreg} should have number of observations equal either to
-#' in-sample or to the whole series. If the number of observations in
-#' \code{xreg} is equal to in-sample, then values for the holdout sample are
-#' produced using Naive.
-#' @param xregDo Variable defines what to do with the provided xreg:
-#' \code{"use"} means that all of the data should be used, whilie
-#' \code{"select"} means that a selection using \code{ic} should be done.
-#' \code{"combine"} will be available at some point in future...
-#' @param initialX Vector of initial parameters for exogenous variables.
-#' Ignored if \code{xreg} is NULL.
-#' @param updateX If \code{TRUE}, transition matrix for exogenous variables is
-#' estimated, introducing non-linear interractions between parameters.
-#' Prerequisite - non-NULL \code{xreg}.
-#' @param persistenceX Persistence vector \eqn{g_X}, containing smoothing
-#' parameters for exogenous variables. If \code{NULL}, then estimated.
-#' Prerequisite - non-NULL \code{xreg}.
-#' @param transitionX Transition matrix \eqn{F_x} for exogenous variables. Can
-#' be provided as a vector. Matrix will be formed using the default
-#' \code{matrix(transition,nc,nc)}, where \code{nc} is number of components in
-#' state vector. If \code{NULL}, then estimated. Prerequisite - non-NULL
-#' \code{xreg}.
 #' @param ...  Other non-documented parameters.  For example parameter
 #' \code{model} can accept a previously estimated CES model and use all its
 #' parameters.  \code{FI=TRUE} will make the function produce Fisher
@@ -173,29 +95,9 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #' cumulative error) and Bias coefficient. This is available only when
 #' \code{holdout=TRUE}.
 #' }
-#' @author Ivan Svetunkov, \email{ivan@@svetunkov.ru}
 #' @seealso \code{\link[forecast]{ets}, \link[forecast]{forecast},
 #' \link[stats]{ts}, \link[smooth]{auto.ces}}
-#' @references \itemize{
-#' \item Svetunkov, I., Kourentzes, N. (February 2015). Complex exponential
-#' smoothing. Working Paper of Department of Management Science, Lancaster
-#' University 2015:1, 1-31.
-#' \item Hyndman, R.J., Koehler, A.B., Ord, J.K., and Snyder, R.D. (2008)
-#' Forecasting with exponential smoothing: the state space approach,
-#' Springer-Verlag. \url{http://www.exponentialsmoothing.net}.
-#' \item Svetunkov S. (2012) Complex-Valued Modeling in Economics and Finance.
-#' SpringerLink: Bucher. Springer.
-#' \item Teunter R., Syntetos A., Babai Z. (2011). Intermittent demand:
-#' Linking forecasting to inventory obsolescence. European Journal of
-#' Operational Research, 214, 606-615.
-#' \item Croston, J. (1972) Forecasting and stock control for intermittent
-#' demands. Operational Research Quarterly, 23(3), 289-303.
-#' \item Syntetos, A., Boylan J. (2005) The accuracy of intermittent demand
-#' estimates. International Journal of Forecasting, 21(2),
-#' 303-314.
-#' }
-#' @keywords ces complex exponential smoothing exponential smoothing
-#' forecasting complex variables
+#'
 #' @examples
 #'
 #' y <- rnorm(100,10,3)
@@ -236,7 +138,7 @@ utils::globalVariables(c("silentText","silentGraph","silentLegend","initialType"
 #'
 #' @export ces
 ces <- function(data, seasonality=c("none","simple","partial","full"),
-                initial=c("backcasting","optimal"), A=NULL, B=NULL, ic=c("AICc","AIC","BIC"),
+                initial=c("optimal","backcasting"), A=NULL, B=NULL, ic=c("AICc","AIC","BIC"),
                 cfType=c("MSE","MAE","HAM","MLSTFE","MSTFE","MSEh"),
                 h=10, holdout=FALSE,
                 intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
@@ -631,21 +533,29 @@ CreatorCES <- function(silentText=FALSE,...){
         }
 # Prepare stuff for intermittency selection
         intermittentModelsPool <- c("n","f","c","t","s");
-        intermittentICs <- rep(1e+10,length(intermittentModelsPool));
+        intermittentCFs <- intermittentICs <- rep(NA,length(intermittentModelsPool));
         intermittentModelsList <- list(NA);
-        intermittentICs <- cesValues$bestIC;
+        intermittentICs[1] <- cesValues$bestIC;
+        intermittentCFs[1] <- cesValues$cfObjective;
 
         for(i in 2:length(intermittentModelsPool)){
             intermittentParametersSetter(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentMaker(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentModelsList[[i]] <- CreatorCES(silentText=TRUE);
             intermittentICs[i] <- intermittentModelsList[[i]]$bestIC;
-            if(intermittentICs[i]>intermittentICs[i-1]){
-                break;
+            intermittentCFs[i] <- intermittentModelsList[[i]]$cfObjective;
+            # if(intermittentICs[i]>intermittentICs[i-1]){
+            #     break;
+            # }
+        }
+        intermittentICs[is.nan(intermittentICs) | is.na(intermittentICs)] <- 1e+100;
+        intermittentCFs[is.nan(intermittentCFs) | is.na(intermittentCFs)] <- 1e+100;
+        # In cases when the data is binary, choose between intermittent models only
+        if(any(intermittentCFs==0)){
+            if(all(intermittentCFs[2:length(intermittentModelsPool)]==0)){
+                intermittentICs[1] <- Inf;
             }
         }
-        intermittentICs[is.nan(intermittentICs)] <- 1e+100;
-        intermittentICs[is.na(intermittentICs)] <- 1e+100;
         iBest <- which(intermittentICs==min(intermittentICs));
 
         if(!silentText){
