@@ -21,7 +21,17 @@ utils::globalVariables(c("measurementEstimate","transitionEstimate", "C",
 #' \code{orders}) and \eqn{l} is a vector of \code{lags}, \eqn{x_t} vector of
 #' exogenous parameters.
 #'
-#' @param data The data that needs to be forecasted.
+#'
+#' @template ssBasicParam
+#' @template ssAdvancedParam
+#' @template ssInitialParam
+#' @template ssPersistenceParam
+#' @template ssAuthor
+#' @template ssKeywords
+#'
+#' @template smoothRef
+#' @template ssIntervalsRef
+#'
 #' @param orders Order of the model. Specified as vector of number of states
 #' with different lags. For example, \code{orders=c(1,1)} means that there are
 #' two states: one of the first lag type, the second of the second type.
@@ -30,100 +40,12 @@ utils::globalVariables(c("measurementEstimate","transitionEstimate", "C",
 #' model will have two states: the first will have lag 1 and the second will
 #' have lag 12. The length of \code{lags} must correspond to the length of
 #' \code{orders}.
-#' @param persistence Persistence vector \eqn{g}, containing smoothing
-#' parameters. If \code{NULL}, then estimated.
 #' @param transition Transition matrix \eqn{F}. Can be provided as a vector.
 #' Matrix will be formed using the default \code{matrix(transition,nc,nc)},
 #' where \code{nc} is the number of components in state vector. If \code{NULL},
 #' then estimated.
 #' @param measurement Measurement vector \eqn{w}. If \code{NULL}, then
 #' estimated.
-#' @param initial Can be either character or a vector of initial states. If it
-#' is character, then it can be \code{"optimal"}, meaning that the initial
-#' states are optimised, or \code{"backcasting"}, meaning that the initials are
-#' produced using backcasting procedure.
-#' @param ic Information criterion to use in model selection.
-#' @param cfType Type of Cost Function used in optimization. \code{cfType} can
-#' be: \code{MSE} (Mean Squared Error), \code{MAE} (Mean Absolute Error),
-#' \code{HAM} (Half Absolute Moment), \code{MLSTFE} - Mean Log Squared Trace
-#' Forecast Error, \code{MSTFE} - Mean Squared Trace Forecast Error and
-#' \code{MSEh} - optimisation using only h-steps ahead error. If
-#' \code{cfType!="MSE"}, then likelihood and model selection is done based on
-#' equivalent \code{MSE}. Model selection in this cases becomes not optimal.
-#'
-#' There are also available analytical approximations for multistep functions:
-#' \code{aMSEh}, \code{aMSTFE} and \code{aMLSTFE}. These can be useful in cases
-#' of small samples.
-#' @param h The forecasting horizon.
-#' @param holdout If \code{TRUE}, holdout of size \code{h} is taken from the
-#' end of the data.
-#' @param intervals Type of intervals to construct. This can be:
-#'
-#' \itemize{
-#' \item \code{none}, aka \code{n} - do not produce prediction
-#' intervals.
-#' \item \code{parametric}, \code{p} - use state-space structure of ETS. In
-#' case of mixed models this is done using simulations, which may take longer
-#' time than for the pure additive and pure multiplicative models.
-#' \item \code{semiparametric}, \code{sp} - intervals based on covariance
-#' matrix of 1 to h steps ahead errors and assumption of normal / log-normal
-#' distribution (depending on error type).
-#' \item \code{nonparametric}, \code{np} - intervals based on values from a
-#' quantile regression on error matrix (see Taylor and Bunn, 1999). The model
-#' used in this process is e[j] = a j^b, where j=1,..,h.
-#' %\item Finally \code{asymmetric} are based on half moment of distribution.
-#' }
-#'
-#' The parameter also accepts \code{TRUE} and \code{FALSE}. Former means that
-#' parametric intervals are constructed, while latter is equivalent to
-#' \code{none}.
-#' @param level Confidence level. Defines width of prediction interval.
-#' @param intermittent Defines type of intermittent model used. Can be: 1.
-#' \code{none}, meaning that the data should be considered as non-intermittent;
-#' 2. \code{fixed}, taking into account constant Bernoulli distribution of
-#' demand occurancies; 3. \code{croston}, based on Croston, 1972 method with
-#' SBA correction; 4. \code{tsb}, based on Teunter et al., 2011 method. 5.
-#' \code{auto} - automatic selection of intermittency type based on information
-#' criteria. The first letter can be used instead. 6. \code{"sba"} -
-#' Syntetos-Boylan Approximation for Croston's method (bias correction)
-#' discussed in Syntetos and Boylan, 2005.
-#' @param bounds What type of bounds to use for smoothing parameters
-#' ("admissible" or "usual"). The first letter can be used instead of the whole
-#' word.
-#' @param silent If \code{silent="none"}, then nothing is silent, everything is
-#' printed out and drawn. \code{silent="all"} means that nothing is produced or
-#' drawn (except for warnings). In case of \code{silent="graph"}, no graph is
-#' produced. If \code{silent="legend"}, then legend of the graph is skipped.
-#' And finally \code{silent="output"} means that nothing is printed out in the
-#' console, but the graph is produced. \code{silent} also accepts \code{TRUE}
-#' and \code{FALSE}. In this case \code{silent=TRUE} is equivalent to
-#' \code{silent="all"}, while \code{silent=FALSE} is equivalent to
-#' \code{silent="none"}. The parameter also accepts first letter of words ("n",
-#' "a", "g", "l", "o").
-#' @param xreg Vector (either numeric or time series) or matrix (or data.frame)
-#' of exogenous variables that should be included in the model. If matrix
-#' included than columns should contain variables and rows - observations. Note
-#' that \code{xreg} should have number of observations equal either to
-#' in-sample or to the whole series. If the number of observations in
-#' \code{xreg} is equal to in-sample, then values for the holdout sample are
-#' produced using Naive.
-#' @param xregDo Variable defines what to do with the provided xreg:
-#' \code{"use"} means that all of the data should be used, whilie
-#' \code{"select"} means that a selection using \code{ic} should be done.
-#' \code{"combine"} will be available at some point in future...
-#' @param initialX Vector of initial parameters for exogenous variables.
-#' Ignored if \code{xreg} is NULL.
-#' @param updateX If \code{TRUE}, transition matrix for exogenous variables is
-#' estimated, introducing non-linear interractions between parameters.
-#' Prerequisite - non-NULL \code{xreg}.
-#' @param persistenceX Persistence vector \eqn{g_X}, containing smoothing
-#' parameters for exogenous variables. If \code{NULL}, then estimated.
-#' Prerequisite - non-NULL \code{xreg}.
-#' @param transitionX Transition matrix \eqn{F_x} for exogenous variables. Can
-#' be provided as a vector. Matrix will be formed using the default
-#' \code{matrix(transition,nc,nc)}, where \code{nc} is number of components in
-#' state vector. If \code{NULL}, then estimated. Prerequisite - non-NULL
-#' \code{xreg}.
 #' @param ...  Other non-documented parameters.  For example parameter
 #' \code{model} can accept a previously estimated GES model and use all its
 #' parameters.  \code{FI=TRUE} will make the function produce Fisher
@@ -184,16 +106,7 @@ utils::globalVariables(c("measurementEstimate","transitionEstimate", "C",
 #' }
 #' @seealso \code{\link[forecast]{ets}, \link[smooth]{es}, \link[smooth]{ces},
 #' \link[smooth]{sim.es}}
-#' @references \enumerate{
-#' \item Teunter R., Syntetos A., Babai Z. (2011). Intermittent demand:
-#' Linking forecasting to inventory obsolescence. European Journal of
-#' Operational Research, 214, 606-615.
-#' \item Croston, J. (1972) Forecasting and stock control for intermittent
-#' demands. Operational Research Quarterly, 23(3), 289-303.
-#' \item Syntetos, A., Boylan J. (2005) The accuracy of intermittent demand
-#' estimates. International Journal of Forecasting, 21(2), 303-314.
-#' }
-#' @keywords ges Exponential Smoothing
+#'
 #' @examples
 #'
 #' # Something simple:
@@ -589,21 +502,29 @@ CreatorGES <- function(silentText=FALSE,...){
         }
 # Prepare stuff for intermittency selection
         intermittentModelsPool <- c("n","f","c","t","s");
-        intermittentICs <- rep(1e+10,length(intermittentModelsPool));
+        intermittentCFs <- intermittentICs <- rep(NA,length(intermittentModelsPool));
         intermittentModelsList <- list(NA);
-        intermittentICs <- gesValues$icBest;
+        intermittentICs[1] <- gesValues$icBest;
+        intermittentCFs[1] <- gesValues$cfObjective;
 
         for(i in 2:length(intermittentModelsPool)){
             intermittentParametersSetter(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentMaker(intermittent=intermittentModelsPool[i],ParentEnvironment=environment());
             intermittentModelsList[[i]] <- CreatorGES(silentText=TRUE);
             intermittentICs[i] <- intermittentModelsList[[i]]$icBest;
-            if(intermittentICs[i]>intermittentICs[i-1]){
-                break;
+            intermittentCFs[i] <- intermittentModelsList[[i]]$cfObjective;
+            # if(intermittentICs[i]>intermittentICs[i-1]){
+            #     break;
+            # }
+        }
+        intermittentICs[is.nan(intermittentICs) | is.na(intermittentICs)] <- 1e+100;
+        intermittentCFs[is.nan(intermittentCFs) | is.na(intermittentCFs)] <- 1e+100;
+        # In cases when the data is binary, choose between intermittent models only
+        if(any(intermittentCFs==0)){
+            if(all(intermittentCFs[2:length(intermittentModelsPool)]==0)){
+                intermittentICs[1] <- Inf;
             }
         }
-        intermittentICs[is.nan(intermittentICs)] <- 1e+100;
-        intermittentICs[is.na(intermittentICs)] <- 1e+100;
         iBest <- which(intermittentICs==min(intermittentICs));
 
         if(!silentText){
